﻿// <copyright file="ArrayExtensions.cs" company="DotNetLib">
//
// .netlib: http://www.codeplex.com/DotNetLib
//
// This code is released under the New BSD License.
// A copy of this license is available at:
// http://www.codeplex.com/DotNetLib/license
//
// </copyright>

namespace DotNetLib
{
  using System;
  using System.Collections.Generic;
  using System.Linq;
  using System.Text;

  /// <summary>
  /// Extension methods for an <see cref="Array"/>.
  /// </summary>
  public static class ArrayExtensions
  {
    /// <summary>
    /// Indicates whether the specified <see cref="Array"/> is null or has a zero length.
    /// </summary>
    /// <param name="array">The <see cref="Array"/> to check.</param>
    /// <returns><b>True</b> if the <see cref="Array"/> is null or has a zero length; <b>false</b> otherwise.</returns>
    public static bool IsNullOrEmpty(this Array array)
    {
      return array == null || array.Length == 0;
    }

    /// <summary>
    /// Formats the array as a string, each element delimited by a seperator.
    /// </summary>
    /// <typeparam name="T">The <see cref="Type"/> of the array.</typeparam>
    /// <param name="list">The list to format.</param>
    /// <param name="format">The format specifier for each element of the array.</param>
    /// <param name="delimiter">The delimiting string to use.</param>
    /// <returns>The array, formatted as as string, each element delimited by a seperator.</returns>
    public static string ToDelimitedString<T>(this IList<T> list, string format, string delimiter)
    {
      if (list == null || list.Count == 0)
      {
        return string.Empty;
      }

      var builder = new StringBuilder();
      for (int index = 0; index < list.Count; ++index)
      {
        if (index != 0)
        {
          builder.Append(delimiter);
        }

        builder.AppendFormat(format, list[index]);
      }

      return builder.ToString();
    }

    /// <summary>
    /// Converts the given byte array to pure hex digits.  { 0, 255 } becomes 00FF
    /// </summary>
    /// <param name="array">The byte array to convert to hex.</param>
    /// <returns>The hex string representation of the byte array.</returns>
    public static string ToHexString(this byte[] array)
    {
      if (IsNullOrEmpty(array))
      {
        return string.Empty;
      }

      var hex = new StringBuilder(array.Length * 2);
      foreach (var b in array)
      {
        hex.AppendFormat("{0:x2}", b);
      }

      return hex.ToString();
    }

    /// <summary>
    /// Combines a series of arrays of the same type into a single array.
    /// </summary>
    /// <typeparam name="T">The <see cref="Type"/> of the array.</typeparam>
    /// <param name="arrays">The arrays to combine.</param>
    /// <returns>A single array of type T, with all of the contents of the arrays.</returns>
    public static T[] Combine<T>(params T[][] arrays)
    {
      if (arrays == null)
      {
        return null;
      }

      var rv = new T[arrays.Sum(a => a.Length)];
      var offset = 0;
      foreach (T[] array in arrays)
      {
        Array.Copy(array, 0, rv, offset, array.Length);
        offset += array.Length;
      }

      return rv;
    }
  }
}
