﻿// <copyright file="CheckBoxListRequiredFieldValidator.cs" company="DotNetLib">
//
// .netlib: http://www.codeplex.com/DotNetLib
//
// This code is released under the New BSD License.
// A copy of this license is available at:
// http://www.codeplex.com/DotNetLib/license
//
// </copyright>

namespace DotNetLib.Web.UI.WebControls
{
  using System;
  using System.Collections.Generic;
  using System.ComponentModel;
  using System.Web;
  using System.Web.UI;
  using System.Web.UI.WebControls;
  using DotNetLib.Internal;

  /// <summary>
  /// Evaluates the values of a <see cref="CheckBoxList"/> to ensure the user selects the appropriate values.
  /// </summary>
  [ToolboxData("<{0}:CheckBoxListRequiredFieldValidator runat=\"server\" ErrorMessage=\"CheckBoxListRequiredFieldValidator\"></{0}:CheckBoxListRequiredFieldValidator>")]
  public class CheckBoxListRequiredFieldValidator : BaseValidator, IScriptControl
  {
    private ScriptManager scriptManager;
    private int minimumRequired = 1;
    private int? maximumAllowed;

    /// <summary>
    /// Gets or sets the minimum number of checkboxes required to be checked.
    /// </summary>
    [SRCategory("Category_Behavior")]
    [DefaultValue(1)]
    [Description("The minimum number of checkboxes that are required to be checked.")]
    public int MinimumRequired
    {
      get { return this.minimumRequired; }
      set { this.minimumRequired = value; }
    }

    /// <summary>
    /// Gets or sets the maximum number of checkboxes allowed to be checked.
    /// </summary>
    [SRCategory("Category_Behavior")]
    [DefaultValue(null)]
    [Description("The maximum number of checkboxes allowed to be checked.")]
    public int? MaximumAllowed
    {
      get { return this.maximumAllowed; }
      set { this.maximumAllowed = value; }
    }

    /// <summary>
    /// Gets or sets the input control to validate.
    /// </summary>
    [Description("The CheckBoxList to validate.")]
    [TypeConverter(typeof(Design.CheckBoxListConverter))]
    public new string ControlToValidate
    {
      get { return base.ControlToValidate; }
      set { base.ControlToValidate = value; }
    }

    IEnumerable<ScriptDescriptor> IScriptControl.GetScriptDescriptors()
    {
      ScriptControlDescriptor descriptor = new ScriptControlDescriptor("DotNetLib.CheckBoxListRequiredFieldValidator", ClientID);

      descriptor.AddProperty("MinimumRequired", this.MinimumRequired);
      descriptor.AddProperty("MaximumAllowed", this.MaximumAllowed);

      yield return descriptor;
    }

    IEnumerable<ScriptReference> IScriptControl.GetScriptReferences()
    {
      ScriptReference reference = new ScriptReference();

      reference.Assembly = "DotNetLib";
      reference.Name = "DotNetLib.Web.UI.WebControls.Validators.js";

      yield return reference;
    }

    /// <summary>
    /// This is a check of properties to determine any errors made by the developer.
    /// </summary>
    /// <returns><b>True</b> if the control properties are valid; otherwise, <b>false</b>.</returns>
    protected override bool ControlPropertiesValid()
    {
      if (string.IsNullOrEmpty(this.ControlToValidate))
      {
        throw new HttpException("ControlToValidate is blank");
      }

      CheckBoxList list = NamingContainer.FindControl(this.ControlToValidate) as CheckBoxList;
      if (list == null)
      {
        throw new NotSupportedException("ControlToValidate is not a CheckBoxList");
      }

      return true;
    }

    /// <summary>
    /// Determines whether the content in the input control is valid.
    /// </summary>
    /// <returns><b>True</b> if the control is valid; otherwise, <b>false</b>.</returns>
    protected override bool EvaluateIsValid()
    {
      CheckBoxList list = NamingContainer.FindControl(this.ControlToValidate) as CheckBoxList;

      int count = 0;
      foreach (ListItem item in list.Items)
      {
        if (item.Selected)
        {
          count++;
        }
      }

      if (count < this.minimumRequired)
      {
        return false;
      }

      if (this.maximumAllowed.HasValue && count > this.maximumAllowed.Value)
      {
        return false;
      }

      return true;
    }

    /// <summary>
    /// Raises the <see cref="Control.PreRender"/> event.
    /// </summary>
    /// <param name="e">An <see cref="EventArgs"/> that contains the event data.</param>
    protected override void OnPreRender(EventArgs e)
    {
      if (!DesignMode && DetermineRenderUplevel())
      {
        this.scriptManager = ScriptManager.GetCurrent(Page);
        if (this.scriptManager == null)
        {
          throw new HttpException("A ScriptManager control must exist on the current page.");
        }

        this.scriptManager.RegisterScriptControl(this);
      }

      base.OnPreRender(e);
    }

    /// <summary>
    /// Displays the control on the client.
    /// </summary>
    /// <param name="writer">An <see cref="HtmlTextWriter"/> that contains the output stream for rendering on the client.</param>
    protected override void Render(HtmlTextWriter writer)
    {
      if (!DesignMode && DetermineRenderUplevel())
      {
        this.scriptManager.RegisterScriptDescriptors(this);
      }

      base.Render(writer);
    }
  }
}
