﻿// <copyright file="SingleProgramInstance.cs" company="DotNetLib">
//
// .netlib: http://www.codeplex.com/DotNetLib
//
// Portions of this were part of the CodePlex "Extensions for .NET Framework"
//    project and written by Marco Minerva, mailto:marco.minerva@gmail.com
//
// This code is released under the New BSD License.
// A copy of this license is available at:
// http://www.codeplex.com/DotNetLib/license
//
// </copyright>

namespace DotNetLib.Windows
{
  using System;
  using System.Diagnostics;
  using System.Reflection;
  using System.Runtime.InteropServices;
  using System.Threading;

  /// <summary>
  /// <see cref="SingleProgramInstance"/> uses a mutex synchronization object to ensure that only one copy of process is running at
  /// a particular time. It also allows for UI identification of the intial process by bring that window to the foreground.
  /// </summary>
  public sealed class SingleProgramInstance : IDisposable
  {
    private Mutex processSync;

    /// <summary>
    /// Initializes a new instance of the <see cref="SingleProgramInstance"/> class.
    /// </summary>
    public SingleProgramInstance()
      : this(string.Empty)
    {
    }

    /// <summary>
    /// Initializes a new instance of the <see cref="SingleProgramInstance"/> class using the specified string as the process identifier.
    /// </summary>
    /// <param name="identifier">The string that represents the process identifier.</param>
    public SingleProgramInstance(string identifier)
    {
      this.processSync = new Mutex(false, Assembly.GetEntryAssembly().GetName().Name + identifier);
    }

    /// <summary>
    /// Gets a value indicating whether this instance is single instance.
    /// </summary>
    /// <value>
    ///   <c>true</c> if this instance is single instance; otherwise, <c>false</c>.
    /// </value>
    public bool IsSingleInstance
    {
      get
      {
        if (this.processSync.WaitOne(0, false))
        {
          return true;
        }
        else
        {
          return false;
        }
      }
    }

    /// <summary>
    /// If another istance of the application is already running, raises the other process.
    /// </summary>
    public void RaiseOtherProcess()
    {
      Process proc = Process.GetCurrentProcess();

      // Using Process.ProcessName does not function properly when
      // the name exceeds 15 characters. Using the assembly name
      // takes care of this problem and is more accruate than other
      // work arounds.
      string assemblyName = Assembly.GetEntryAssembly().GetName().Name;
      foreach (Process otherProc in Process.GetProcessesByName(assemblyName))
      {
        // ignore this process
        if (proc.Id != otherProc.Id)
        {
          // Found a "same named process".
          // Assume it is the one we want brought to the foreground.
          // Use the Win32 API to bring it to the foreground.
          if (NativeMethods.IsIconic(otherProc.MainWindowHandle))
          {
            NativeMethods.ShowWindowAsync(otherProc.MainWindowHandle, NativeMethods.SW_RESTORE);
          }

          NativeMethods.SetForegroundWindow(otherProc.MainWindowHandle);
          return;
        }
      }
    }

    #region Implementation of IDisposable

    /// <summary>
    /// Finalizes an instance of the <see cref="SingleProgramInstance"/> class.
    /// </summary>
    ~SingleProgramInstance()
    {
      // Release mutex (if necessary) 
      // This should have been accomplished using Dispose() 
      this.FreeResources();
    }

    private void FreeResources()
    {
      try
      {
        if (this.processSync.WaitOne(0, false))
        {
          // If we own the mutex than release it so that
          // other "same" processes can now start.
          this.processSync.ReleaseMutex();
        }

        this.processSync.Close();
      }
      catch
      {
      }
    }

    /// <summary>
    /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
    /// </summary>
    public void Dispose()
    {
      // release mutex (if necessary) and notify 
      // the garbage collector to ignore the destructor
      this.FreeResources();
      GC.SuppressFinalize(this);
    }

    #endregion
  }
}
